class List:
    def __init__(self):
        self.list = []

    def run(self, vars, parts):
        for elem in self.list:
            if type(elem) is type(''):
                parts.append(elem)
            else:
                elem.run(vars, parts)

class Eval:
    def __init__(self, expr):
        self.expr = expr

    def run(self, vars, parts):
        text = eval(self.expr, vars)
        if text:
            parts.append(text)

class Exec:
    def __init__(self, expr):
        self.expr = expr

    def run(self, vars, parts):
        exec self.expr in vars

class Include:
    def __init__(self, filename):
        self.templ = FileTempl(filename)

    def run(self, vars, parts):
        self.templ.run(vars, parts)

class If(List):
    def __init__(self, expr):
        List.__init__(self)
        self.expr = expr

    def run(self, vars, parts):
        if eval(self.expr, vars):
            List.run(self, vars, parts)

class For(List):
    def __init__(self, var, expr):
        List.__init__(self)
        self.var = var
        self.expr = expr

    def run(self, vars, parts):
        seq = eval(self.expr, vars)
        for elem in seq:
            vars[self.var] = elem
            List.run(self, vars, parts)

class StrTempl:
    def __init__(self, text):
        in_content = 1
        self.prog = List()
        self.curr = self.prog.list
        self.stack = []
        for part in text.split('$'):
            if in_content:
                self.curr.append(part)
                in_content = 0
                continue
            if len(part) < 2 or part[0] != part[-1]:
                self.curr.append('$')
                self.curr.append(part)
                continue
            self._add_code(part)
            in_content = 1

    def run(self, vars, parts = None):
        do_join = parts is None
        if do_join:
            parts = []
        self.prog.run(vars, parts)
        if do_join:
            return ''.join(parts)

    def _add_code(self, part):
        if part[0] == '=':
            self.curr.append(Eval(part[1:-1]))
        if part[0] == '#':
            return
        if part[0] == '%':
            words = part[1:-1].split()
            if len(words) == 1 and words[0] == 'end':
                self._pop()
            if words[0] == 'for':
                self._push(For(words[1], ' '.join(words[2:])))
            if words[0] == 'if':
                self._push(If(' '.join(words[1:])))
            if words[0] == 'include':
                self.curr.append(Include(' '.join(words[1:])))
            if words[0] == 'exec':
                self.curr.append(Exec(' '.join(words[1:])))

    def _pop(self):
        self.curr = self.stack[-1]
        del self.stack[-1]

    def _push(self, elem):
        self.stack.append(self.curr)
        self.curr.append(elem)
        self.curr = elem.list

class FileTempl(StrTempl):
    def __init__(self, filename):
        fp = open(filename)
        text = fp.read()
        fp.close()
        StrTempl.__init__(self, text)
